function T = bclust2sptens(A, B, C)
% BCLUST2SPTENS turns a Boolean clustering in to an sptensor
%
% Usage:
%   T = BCLUST2SPTENS(A, B, C)
%   where A and B are two binary matrices and C is either a cluster
%   assignment array or a binary cluster assignment matrix.
%
% See also: SPTENSOR

[n, k] = size(A);
[m, foo] = size(B);

if foo ~= k
    error('Inner dimensions of A and B do not match');
end

if ~isvector(C)
    nrClust = size(C, 2);
    clusters = nan(size(C,1), 1);
    for i=1:nrClust
        I = find(C(:, i));
        clusters(I) = i;
    end
    C = clusters;
else
    if size(C, 2) > size(C, 1)
        C = C';
    end
    nrClust = max(C);
end

if nrClust > k
    error(['Number of clusters does not agree with the number of ' ...
           'centroids']);
end

centroids = cell(k, 1);
for i=1:k,
    [I, J] = find(A(:,i)*B(:,i)');
    centroids{i} = [I, J];
end

centroidSizes = cellfun(@(x) size(x,1), centroids);
tensorNNZ = sum(arrayfun(@(x) centroidSizes(x), C));

loc_nnz = nan(tensorNNZ, 3);
loc_nnz_row = 1;

for slice=1:length(C)
    centroid = centroids{C(slice)};
    cS = centroidSizes(C(slice));
    s = repmat(slice, cS, 1); 
    loc_nnz(loc_nnz_row:loc_nnz_row+cS-1, :) = [centroid s];
    loc_nnz_row = loc_nnz_row + cS;
end

T = sptensor(loc_nnz, 1, [n, m, length(C)]);

