/*
 * matlabToBitwise.h
 *
 *  Created on: Jul 1, 2014
 *      Author: smetzler
 */

#ifndef BTC_H_
#define BTC_H_

#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <limits.h>

#ifndef POPCNT64
#define POPCNT64(x) __builtin_popcountll(x) //_popcnt64
#endif

//#define TESTMODE

typedef struct{
	uint64_t x;
	uint64_t y;
	uint64_t z;
} dim3;

typedef struct{
	uint64_t x;
	uint64_t y;
} dim2;

typedef struct {
	double* data;
	uint64_t length;
} arrayWithLength;

typedef struct {
	uint64_t* assignment;
	uint64_t similarity;
} clusterAssignment;

typedef struct {
	uint64_t* tensor;
	uint64_t* onesPerColumn;
} bitwiseTensor;

typedef struct {
	uint64_t* A;
	uint64_t* B;
	uint64_t* C;
	uint64_t simi;
} decom3;

typedef struct {
	uint64_t* A;
	uint64_t* B;
} decom;

typedef struct {
	uint64_t* centroids;
	uint64_t* assignment;
	uint64_t similarity;
} clustering;

static const uint64_t ONE = 1;
static const int MAX_ITERATIONS = 15;

uint64_t* randomPermutation(const uint64_t n);
uint64_t* range(const uint64_t n, const uint64_t offset);

decom rank1approx(const uint64_t* restrict A, const uint64_t* restrict colsums, const dim2 dim);
decom rank1approx_parallel(const uint64_t* restrict A, const uint64_t* restrict colsums, const dim2 dim);

decom3 btc(const bitwiseTensor* X, const dim3 dim, const uint64_t numberOfClusters, const uint64_t reps, const uint64_t numThreads);
decom3 btc_parallel(const bitwiseTensor* X, const dim3 dim, const uint64_t numberOfClusters, const uint64_t reps, const uint64_t numThreads);

uint64_t* toRank1ProductMatrix(const bitwiseTensor* A, const dim3 dim);
uint64_t* checkedCalloc(const uint64_t numberOfElements);
void printBinary(const uint64_t n);
void printTensorColWise(const bool* tensor, const dim3 dim);
void printBitwiseTensor(const uint64_t* tensor, const dim3 dim);
void printSparseBinaryTensor(const double* data, size_t numberOfRows);
bitwiseTensor* toBitRepresentation(const bool* matlabTensor, const dim3 dim);
bool* toMatlabRepresentation(const uint64_t* bitwiseTensor , const dim3 dim);
bitwiseTensor* sparseToBitRepresentation(const double* coordinates, const uint64_t len, const dim3 dim);
arrayWithLength* toSparseMatlabRepresentation(const uint64_t* bitwiseTensor , const dim3 dim);

void iterativeUpdates(const bitwiseTensor* X, clustering* clustering, const dim3 dim, uint64_t numberOfClusters, bool rank1);
clustering getInitialClustering(const bitwiseTensor* X, const dim3 dim, const uint64_t numberOfClusters, const uint64_t reps, const uint64_t numThreads, const bool rank1Centroids);
uint64_t* outerProduct(const uint64_t* A, const uint64_t* B, const dim2 dim);

uint64_t numberOfUint64s(const uint64_t n);
uint64_t bitwiseTensorSize(const dim3 dim);
uint64_t prod(const dim3 dim);

#endif /* BTC_H_ */
