#include <string.h>
#include "mex.h"
#include "btc.h"
//#include <simstruc.h> // needed to forward printf output to matlab

/* Given a binary 3-way tensor, returns a clustering.
 *
 * This function is intended to be called from Matlab.
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	if (!(nrhs == 6 || nrhs == 5))
        mexErrMsgTxt("Wrong number of input arguments. Three to four arguments expected.");
    if (nlhs != 3)
        mexErrMsgTxt("Wrong number of output arguments. Exactly four arguments expected.");

	const double const *mdim = mxGetData(prhs[1]);
    const dim3 dim = { .x = mdim[0], .y = mdim[1], .z = mdim[2]};
    const mwSize k = mxGetScalar(prhs[2]);
    const mwSize repetitions = mxGetScalar(prhs[3]);
    const bool rank1Centroids = (bool) mxGetScalar(prhs[4]);
    const mwSize numThreads = (nrhs == 6) ? mxGetScalar(prhs[5]) : 0;
    const size_t m = mxGetM(prhs[0]);
    double* coordinates = mxGetData(prhs[0]);

    /* Convert to bitwise representation and run clustering. */
    bitwiseTensor* bitPl = sparseToBitRepresentation(coordinates, m, dim);

    clustering c = getInitialClustering(bitPl,dim,k,repetitions,numThreads,rank1Centroids);
	iterativeUpdates(bitPl,&c,dim,k,rank1Centroids);

    free(bitPl->tensor);
    free(bitPl->onesPerColumn);
    free(bitPl);

    /* Prepare output. (Dense!) */
	const mwSize dimCentroids[3] = {dim.x, dim.y, k};
    const mwSize dimAssignment[2] = {dim.z, k};

    mxLogical* centroids = (mxLogical*) toMatlabRepresentation(
    		c.centroids, (dim3) {dim.x,dim.y,k});

    plhs[0] = mxCreateLogicalArray(3, dimCentroids);
    mxLogical *pnA = mxGetLogicals(plhs[0]);
    memcpy(pnA, centroids, dim.x*dim.y*k*sizeof(mxLogical));

    mxLogical* assignment = (mxLogical*) toMatlabRepresentation(
    		c.assignment, (dim3) {dim.z,k,1});

    plhs[1] = mxCreateLogicalArray(2, dimAssignment);
    mxLogical *pnC = mxGetLogicals(plhs[1]);
    memcpy(pnC, assignment, dim.z*k*sizeof(mxLogical));

    plhs[2] = mxCreateDoubleScalar(c.similarity);
    return;
}
