function run_iterative_updates_overfitting_test
    addpath(genpath('.'))
    datadir = '../experiments/synth/data/';
    resdir = '../experiments/synth/overfitting';
    algname = 'iterative_updates';

    n = 700;
    m = 500;
    l = 300;
    o = 225;

    k = 7;
    d = 0.15;

    addNoise = 0.1:0.1:0.5;
    desNoise = 0.1:0.1:0.5;
    defAddNoise = 0.1;
    defDesNoise = 0.1;

    reps = 10;
    iters = 5;

    %% Noise, both types

    for rank1centroids=[false,true]
        simiTrain = nan(iters, length(addNoise));
        simiTest = nan(iters, length(addNoise));
        origSimiTest = nan(iters, length(addNoise));
        Time = nan(iters, length(addNoise));
        j = 0;
        for noise=addNoise
            fprintf(2, 'noise = %.2f', noise);
            j = j + 1;
            for i=1:iters
                fprintf(2, '.');
                [simiTrain(i, j), simiTest(i, j), origSimiTest(i, j), Time(i, j)] = run_experiment (...
                    datadir, 'overfitting', resdir, algname, n, m, l, o, k, d, ...
                    noise, noise, reps, i, rank1centroids);
            end
            fprintf(2, '\n');
            save(sprintf('%s/noise_%s_%s.mat', resdir, algname, ...
                conditional(rank1centroids, 'rank1', 'full')), ...
                'simiTrain', 'simiTest', 'origSimiTest', 'Time');
        end
    end


    %% additive Noise
    dnoise = defDesNoise;
    
    for rank1centroids=[false,true]
        simiTrain = nan(iters, length(addNoise));
        simiTest = nan(iters, length(addNoise));
        origSimiTest = nan(iters, length(addNoise));
        Time = nan(iters, length(addNoise));
        j = 0;
        for anoise=addNoise
            fprintf(2, 'anoise = %.2f', anoise);
            j = j + 1;
            for i=1:iters
                fprintf(2, '.');
                [simiTrain(i, j), simiTest(i, j), origSimiTest(i, j), Time(i, j)] = run_experiment (...
                    datadir, 'overfitting', resdir, algname, n, m, l, o, k, d, ...
                    dnoise, anoise, reps, i, rank1centroids);
            end
            fprintf(2, '\n');
            save(sprintf('%s/addNoise_%s_%s.mat', resdir, algname, ...
                conditional(rank1centroids, 'rank1', 'full')), ...
                'simiTrain', 'simiTest', 'origSimiTest', 'Time');
        end
    end
    
    %% destructive Noise
    anoise = defAddNoise;
    
    for rank1centroids=[false,true]
        simiTrain = nan(iters, length(desNoise));
        simiTest = nan(iters, length(desNoise));
        origSimiTest = nan(iters, length(desNoise));
        Time = nan(iters, length(desNoise));
        j = 0;
        for dnoise=desNoise
            fprintf(2, 'dnoise = %.2f', dnoise);
            j = j + 1;
            for i=1:iters
                fprintf(2, '.');
                [simiTrain(i, j), simiTest(i, j), origSimiTest(i, j), Time(i, j)] = run_experiment (...
                    datadir, 'overfitting', resdir, algname, n, m, l, o, k, d, ...
                    dnoise, anoise, reps, i, rank1centroids);
            end
            fprintf(2, '\n');
            save(sprintf('%s/desNoise_%s_%s.mat', resdir, algname, ...
                conditional(rank1centroids, 'rank1', 'full')), ...
                'simiTrain', 'simiTest', 'origSimiTest', 'Time');
        end
    end
end    
        
function [simiTrain, simiTest, origSimTest, time] = run_experiment (datadir, ...
    prefix, resdir, algname, n, m, l, o, k, d, dnoise, anoise, reps, i, rank1centroids)
    % Trains k-means on o slices and evaluates on full tensor
    % The test is done twice, once with rank-1 centroids and once with full
    
    load(sprintf('%s%s_n%i_m%i_l%i_k%i_d%.2f_desNoise%.2f_addNoise%.2f_iter%i.mat', ...
        datadir, prefix, n, m, l, k, d, dnoise, anoise, i));

    trainSlices = randperm(l, o);
    testSlices = setdiff(1:l, trainSlices);
    subT = T(:,:,trainSlices);
    
    bestSimTrain = 0;
    bestAssTrain = [];
    bestCentroids = [];
    for i=1:reps
        tic;
        [centroids, assTrain, similarity] = iterative_updates(...
            subT.subs,subT.size,k,1,rank1centroids);
        time = toc;
        simiTrain = similarity/(m*n*o);
        if simiTrain > bestSimTrain
            bestSimTrain = simiTrain;
            bestAssTrain = assTrain;
            bestCentroids = centroids;
        end
    end
    simiTrain = bestSimTrain;
    centroids = bestCentroids;
    assTrain = bestAssTrain;
        
    matT = smatricize(T(:,:,testSlices), 3);
    vecCentroids = transpose(reshape(centroids, m*n, k));
    assTest = zeros(n, 1);
    simi = 0;
    for r=1:length(testSlices)
        dist = sum(bsxfun(@xor, matT(r, :), vecCentroids), 2);
        [err, ind] = min(dist);
        ind = ind(1);
        s = m*n - err;
        simi = simi + s;
        assTest(r) = ind;
    end
    simiTest = simi/(m*n*length(testSlices));
    
    OT = bclust2sptens(A, B, C);
    OT = smatricize(OT(:,:,testSlices), 3);
    origSimTest = m*n*length(testSlices);
    for slice=1:length(testSlices)
        centroid = vecCentroids(assTest(slice),:);
        origSimTest = origSimTest - nnz(xor(OT(slice,:), centroid));
    end
    origSimTest = origSimTest / (m*n*length(testSlices));
        
    save(sprintf('%s/%s_overfitting_test_n%i_m%i_l%i_o%i_k%i_d%.2f_desNoise%.2f_addNoise%.2f_%s.mat', ...
        resdir, algname, n, m, l, o, k, d, dnoise, anoise, conditional(rank1centroids, 'rank1', 'full')), ...
        'centroids', 'assTrain', 'assTest', 'testSlices', 'simiTrain', 'simiTest', 'origSimTest', 'time');
end

function c = conditional(condition , a , b)
    if condition
        c = a;
    else
        c = b;
    end
end