% Runs synthetic experiments
function run_synth_exps_bitwise_btc_sparse
    resdir = '../experiments/synth/';
    reps = 20;
    iters = 5;

    datadir = '../experiments/synth/data_density5percent/';
    algname = 'bitwise_btc_density5percent';

    n = 700;
    m = 500;
    l = 50;

    K = 3:2:15;
    D = 0.05:0.05:0.3;
    addNoise = 0.05:0.05:0.5;
    desNoise = 0.05:0.05:0.5;

    defK = 7;
    defD = 0.05;
    defAddNoise = 0.1;
    defDesNoise = 0.1;

    %% K
    d = defD;
    dnoise = defDesNoise;
    anoise = defAddNoise;

    S = nan(iters, length(K));
    SO = nan(iters, length(K));
    Time = nan(iters, length(K));
    j = 0;

    for k=K
        fprintf(2, 'k = %i ', k);
        j = j + 1; 
        for i=1:iters
            fprintf(2, '.');
            [S(i, j), SO(i, j), Time(i, j)] = run_experiment (datadir, ...
                'k', resdir, algname, n, m, l, k, d, dnoise, anoise, reps, i);
        end
        fprintf(2, '\n');
        save(sprintf('%sk_%s.mat', resdir, algname), 'S', 'SO', 'Time');
    end


    %% D
    k = defK;
    dnoise = defDesNoise;
    anoise = defAddNoise;

    S = nan(iters, length(D));
    SO = nan(iters, length(D));
    Time = nan(iters, length(D));
    j = 0;

    for d=D
        fprintf(2, 'd = %.2f', d);
        j = j + 1;
        for i=1:iters
            fprintf(2, '.');
            [S(i, j), SO(i, j), Time(i, j)] = run_experiment (datadir, ...
                'd', resdir, algname, n, m, l, k, d, dnoise, anoise, reps, i);
        end
        fprintf(2, '\n');   
        save(sprintf('%sdensity_%s.mat', resdir, algname), 'S', 'SO', 'Time');
    end


    %% destructive Noise
    k = defK;
    d = defD;
    anoise = defAddNoise;

    S = nan(iters, length(desNoise));
    SO = nan(iters, length(desNoise));
    Time = nan(iters, length(desNoise));
    j = 0;

    for dnoise=desNoise
        fprintf(2, 'dnoise = %.2f', dnoise);
        j = j + 1;
        for i=1:iters
            fprintf(2, '.');
            [S(i, j), SO(i, j), Time(i, j)] = run_experiment (datadir, ...
                'desNoise', resdir, algname, n, m, l, k, d, dnoise, anoise, reps, i);
        end
        fprintf(2, '\n');
        save(sprintf('%sdesNoise_%s.mat', resdir, algname), 'S', 'SO', 'Time');
    end
    
    %% additive Noise
    k = defK;
    d = defD;
    dnoise = defDesNoise;
    
    S = nan(iters, length(addNoise));
    SO = nan(iters, length(addNoise));
    Time = nan(iters, length(addNoise));
    j = 0;

    for anoise=addNoise
        fprintf(2, 'anoise = %.2f', anoise);
        j = j + 1;
        for i=1:iters
            fprintf(2, '.');
            [S(i, j), SO(i, j), Time(i, j)] = run_experiment (datadir, ...
                'addNoise', resdir, algname, n, m, l, k, d, dnoise, anoise, reps, i);
        end
        fprintf(2, '\n');
        save(sprintf('%saddNoise_%s.mat', resdir, algname), 'S', 'SO', 'Time');
    end
end
        
function [sim, sim_orig, time] = run_experiment (datadir, prefix, resdir, algname, n, m, l, k, d, dnoise, anoise, reps, i)
        load(sprintf('%s%s_n%i_m%i_l%i_k%i_d%.2f_desNoise%.2f_addNoise%.2f_iter%i.mat', ...
            datadir, prefix, n, m, l, k, d, dnoise, anoise, i));
        tic
        [mA, mB, mC, sim] = sparse_bitwise_btc(T.subs, T.size, k, reps);
        time = toc;
        
        mA = spconvert([mA ones(length(mA),1); n k 0]);
        mB = spconvert([mB ones(length(mB),1); m k 0]);
        mC = spconvert([mC ones(length(mC),1); l k 0]);
        
        %OT = bclust2sptens(A, B, C);
        %AT = bclust2sptens(mA, mB, mC);
        sim_orig = 0; %m*n*l-nnz(xor(OT, AT));
        
        save(sprintf('%s/%s/%s_n%i_m%i_l%i_k%i_d%.2f_desNoise%.2f_addNoise%.2f_iter%i.mat', ...
            resdir, algname, prefix, n, m, l, k, d, dnoise, anoise, i), ...
            'mA', 'mB', 'mC', 'time', 'sim', 'sim_orig');
        clear T mA mB mC OT AT;
end
