function write_tensors( datadir, prefix, n, m, l, k, d, desNoise, addNoise, iterations )
%WRITE_TENSORS writes a number of random sparse tensors 
% with the specified parameters, once as .mat and once as .dat where the 
% latter is a binary that contains the indices as doubles and the last 
% three entries are the dimensionality of the full tensor. Additive and
% destructive noise are with respect to the number of non-zeroes in the
% tensor.
    i = 1;
    while i <= iterations
        A = sparse(double(rand(n, k) < sqrt(d)));
        B = sparse(double(rand(m, k) < sqrt(d)));
        ci = randsample(k, l, true);
        C = sparse(ci2mat(ci, k));

        T = bclust2sptens(A, B, C);

        % adding noise
        myTsubs = T.subs;
        origPositionsOfNonZeroes = sub2ind([n,m,l], T.subs(:,1), T.subs(:,2), T.subs(:,3));
        numberOfNonZeroes = nnz(T);
        numberOfZeroesToAdd = ceil(desNoise*numberOfNonZeroes);
        if numberOfZeroesToAdd > numberOfNonZeroes
            warning('Too high destructive noise requested. Setting number of zeroes to add to maximum possible.');
            numberOfZeroesToAdd = numberOfNonZeroes;
        end
        desNoisePos = randperm(numberOfNonZeroes, numberOfNonZeroes - numberOfZeroesToAdd);
        myTsubs = myTsubs(desNoisePos,:);

        numberOfOnesToAdd = ceil(addNoise * numberOfNonZeroes);
        if numberOfOnesToAdd > m*n*l - numberOfNonZeroes
            warning('Too high additive noise requested. Setting number of zeroes to add to maximum possible.');
            numberOfOnesToAdd = m*n*l - numberOfNonZeroes;
        end
        addNoisePos = randperm(n*m*l, numberOfOnesToAdd + numberOfNonZeroes);
        addNoisePos = setdiff(addNoisePos, origPositionsOfNonZeroes);
        [addNoisePosN, addNoisePosM, addNoisePosL] = ind2sub([n,m,l],addNoisePos);
        maxAddNoise = [addNoisePosN' addNoisePosM' addNoisePosL'];
        myTsubs = [myTsubs; maxAddNoise(1:numberOfOnesToAdd,:)];
        
        if length(unique(myTsubs(:,3))) < l 
            warning('Tensor rejected because of all-zero slice.');
            i = i-1;
        else
            T = sptensor(myTsubs, 1, [n,m,l]);

            save(sprintf('%s%s_n%i_m%i_l%i_k%i_d%.2f_desNoise%.2f_addNoise%.2f_iter%i.mat', ...
                datadir, prefix, n, m, l, k, d, desNoise, addNoise, i), 'A', 'B', 'C', 'T', '-v7.3');
            
            %% saving tensors this way is not suitable for large data
            %fname = sprintf('%s%s_n%i_m%i_l%i_k%i_d%.2f_desNoise%.2f_addNoise%.2f_iter%i.dat', ...
            %    datadir, prefix, n, m, l, k, d, desNoise, addNoise, i);
            %[f,e] = fopen(fname, 'w');
            %fwrite(f, [reshape(T.subs, size(T.subs,1)*size(T.subs,2),1);transpose(size(T))], 'double');
            %fclose(f);
            
            i = i+1;
        end
        clear T;
    end
end

